function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* eslint no-dupe-class-members:0, no-throw-literal:0, no-case-declarations:0 */
import memo from '@bevry/memo';
import startOfWeek from 'start-of-week';
import * as intl from './intl.js';
export var Millisecond = 1;
export var Second = Millisecond * 1000;
export var Minute = Second * 60;
export var Hour = Minute * 60;
export var Day = Hour * 24;
export var Week = Day * 7;
/** The abstract base tier to use for the human relative date display. */

/** A minimal immutable date class that supports relative time, calendar time, and plus/minus of different units. */
var Daet =
/*#__PURE__*/
function () {
  _createClass(Daet, null, [{
    key: "create",

    /** Create a new daet instance based on the input */
    value: function create(input) {
      return new this(input);
    }
    /** Create a new daet instance based on the input */

  }, {
    key: "tiers",

    /** The raw date object behind this daet instance. */

    /** The tiers used for human relative date display. */
    get: function get() {
      return [{
        // right now
        limit: Second,
        refresh: Second,
        message: intl.rightNow
      }, {
        // x seconds
        limit: Minute,
        refresh: Second,
        message: intl.relativeDelta
      }, {
        // x minutes
        limit: Hour,
        refresh: Minute,
        message: intl.relativeDelta
      }, {
        // x hours x minutes
        limit: 12 * Hour,
        refresh: Minute,
        message: intl.relativeDelta
      }, {
        // later/earlier today
        when: function when(_ref) {
          var past = _ref.past;
          return past ? new Daet().reset('hour').getTime() : new Daet().plus(1, 'day').reset('hour').getTime();
        },
        message: intl.earlierOrLaterToday
      }, {
        // yesterday or tomorrow
        when: function when(_ref2) {
          var past = _ref2.past;
          return past ? new Daet().minus(1, 'day').reset('hour').getTime() : new Daet().plus(2, 'day').reset('hour').getTime();
        },
        message: intl.yesterdayOrTommorow
      }, {
        // this week
        when: function when(_ref3) {
          var past = _ref3.past;
          return past ? new Daet().endOfLastWeek().getTime() : new Daet().startOfNextWeek().getTime();
        },
        message: intl.relativeThisWeek
      }, {
        // next or last week
        when: function when(_ref4) {
          var past = _ref4.past;
          return past ? new Daet().endOfLastWeek().endOfLastWeek().getTime() : new Daet().startOfNextWeek().startOfNextWeek().getTime();
        },
        message: intl.relativeSecondWeek
      }, {
        // earlier or later
        limit: Infinity,
        message: intl.earlierOrLater
      }];
    }
  }]);

  function Daet(input) {
    var _this = this;

    _classCallCheck(this, Daet);

    _defineProperty(this, "raw", void 0);

    _defineProperty(this, "getTime", memo(function () {
      return _this.raw.getTime();
    }));

    _defineProperty(this, "startOfNextWeek", memo(function () {
      // continue until we become the start of next week
      var latest = _this.plus(1, 'day');

      while (latest.raw.getDay() !== startOfWeek) {
        latest = latest.plus(1, 'day');
      } // reset the time, so we become the start time of that week


      return latest.reset('hour');
    }));

    _defineProperty(this, "endOfLastWeek", memo(function () {
      // continue until we become the start of this week
      var latest = _this;

      while (latest.raw.getDay() !== startOfWeek) {
        latest = latest.minus(1, 'day');
      } // reset the time, then minus a second, so we become the end of the prior week


      return latest.reset('hour').minus(1, 'second');
    }));

    _defineProperty(this, "toISOString", memo(function () {
      return _this.raw.toISOString();
    }));

    _defineProperty(this, "toJSON", memo(function () {
      return _this.raw.toJSON();
    }));

    this.raw = input instanceof Daet ? new Date(input.raw) : input ? new Date(input) : new Date();
    this.raw.setMilliseconds(0);
  }
  /** Return a new daet instance that is in the past by the value amount. */


  _createClass(Daet, [{
    key: "minus",
    value: function minus(value, unit) {
      return this.plus(value * -1, unit);
    }
    /** Return a new daet instance that is in the future by the value amount. */

  }, {
    key: "plus",
    value: function plus(value, unit) {
      switch (unit) {
        case 'millisecond':
          {
            var next = new Date(this.raw.getTime() + value);
            return new Daet(next);
          }

        case 'second':
          {
            return this.plus(value * Second, 'millisecond');
          }

        case 'minute':
          {
            return this.plus(value * Minute, 'millisecond');
          }

        case 'hour':
          {
            return this.plus(value * Hour, 'millisecond');
          }

        case 'day':
          {
            return this.plus(value * Day, 'millisecond');
          }

        case 'week':
          {
            return this.plus(value * Week, 'millisecond');
          }

        default:
          // https://basarat.gitbooks.io/typescript/docs/types/discriminated-unions.html
          var neverCheck = unit;
          throw new Error('unknown unit');
      }
    }
    /** Clone this daet instance based on its raw value. */

  }, {
    key: "rawClone",
    value: function rawClone() {
      return new Date(this.raw);
    }
    /** Clone this daet instance. */

  }, {
    key: "clone",
    value: function clone() {
      return new Daet(this);
    }
    /** Return a new daet instance that has the unit set to the desired value. */

  }, {
    key: "set",
    value: function set(value, unit) {
      switch (unit) {
        case 'millisecond':
          {
            var next = this.rawClone().setMilliseconds(value);
            return new Daet(next);
          }

        case 'second':
          {
            var _next = this.rawClone().setSeconds(value);

            return new Daet(_next);
          }

        case 'minute':
          {
            var _next2 = this.rawClone().setMinutes(value);

            return new Daet(_next2);
          }

        case 'hour':
          {
            var _next3 = this.rawClone().setHours(value);

            return new Daet(_next3);
          }

        default:
          // https://basarat.gitbooks.io/typescript/docs/types/discriminated-unions.html
          var neverCheck = unit;
          throw new Error('unknown unit');
      }
    }
    /** Return a new daet instance that has the desired unit reset to 0. */

  }, {
    key: "reset",
    value: function reset(unit) {
      switch (unit) {
        case 'millisecond':
          {
            var next = this.rawClone().setMilliseconds(0);
            return new Daet(next);
          }

        case 'second':
          {
            var _next4 = this.rawClone().setSeconds(0, 0);

            return new Daet(_next4);
          }

        case 'minute':
          {
            var _next5 = this.rawClone().setMinutes(0, 0, 0);

            return new Daet(_next5);
          }

        case 'hour':
          {
            var _next6 = this.rawClone().setHours(0, 0, 0, 0);

            return new Daet(_next6);
          }

        default:
          // https://basarat.gitbooks.io/typescript/docs/types/discriminated-unions.html
          var neverCheck = unit;
          throw new Error('unknown unit');
      }
    }
    /** Get the epoch time of this daet instance. */

  }, {
    key: "getMillisecondsFrom",

    /** Get the milliseconds from the passed daet instance to this daet instance. */
    value: function getMillisecondsFrom(from) {
      var now = from.getTime();
      var time = this.getTime();
      return time - now;
    }
    /** Get the milliseconds from now to this daet instance */

  }, {
    key: "getMillisecondsFromNow",
    value: function getMillisecondsFromNow() {
      return this.getMillisecondsFrom(new Daet());
    }
    /** Use https://devdocs.io/javascript/global_objects/datetimeformat to format our daet instance. */

  }, {
    key: "format",
    value: function format(locale, options) {
      return new Intl.DateTimeFormat(locale, options).format(this.raw);
    }
    /** Return a new daet instance that is the start of the week proceeding that of this daet instance. */

  }, {
    key: "calendar",

    /** Get the human absolute date display for this daet instance. */
    value: function calendar() {
      return this.format('en', {
        weekday: 'long',
        year: 'numeric',
        month: 'long',
        day: 'numeric',
        hour: 'numeric',
        minute: 'numeric'
      });
    }
    /** Return the human relative display from now, as well as the millisecond delta before a refresh is needed. */

  }, {
    key: "fromNowDetails",
    value: function fromNowDetails() {
      var now = new Daet();
      var nowTime = now.getTime();
      var eventTime = this.getTime();
      var past = nowTime > eventTime;
      var delta = Math.abs(eventTime - nowTime);
      var tiers = Daet.tiers;
      var lastTierDelta = 0;
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = tiers[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var tier = _step.value;
          var limit = tier.limit;
          var when = tier.when ? tier.when({
            past: past
          }) : 0;
          var tierDelta = limit || Math.abs(when - nowTime);

          if (delta < tierDelta) {
            var message = tier.message({
              past: past,
              delta: delta,
              when: this
            });
            var refresh = tier.refresh || delta - lastTierDelta;
            return {
              message: message,
              refresh: refresh
            };
          }

          lastTierDelta = tierDelta;
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      throw new Error('no tier matched the input delta');
    }
    /** Return the human relative display from now. */

  }, {
    key: "fromNow",
    value: function fromNow() {
      return this.fromNowDetails().message;
    }
    /** Return the ISO string for this daet instance. */

  }]);

  return Daet;
}();

export { Daet as default };