"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.Week = exports.Day = exports.Hour = exports.Minute = exports.Second = exports.Millisecond = void 0;

var _memo = _interopRequireDefault(require("@bevry/memo"));

var _startOfWeek = _interopRequireDefault(require("start-of-week"));

var intl = _interopRequireWildcard(require("./intl.js"));

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const Millisecond = 1;
exports.Millisecond = Millisecond;
const Second = Millisecond * 1000;
exports.Second = Second;
const Minute = Second * 60;
exports.Minute = Minute;
const Hour = Minute * 60;
exports.Hour = Hour;
const Day = Hour * 24;
exports.Day = Day;
const Week = Day * 7;
/** The abstract base tier to use for the human relative date display. */

exports.Week = Week;

/** A minimal immutable date class that supports relative time, calendar time, and plus/minus of different units. */
class Daet {
  /** The raw date object behind this daet instance. */

  /** The tiers used for human relative date display. */
  static get tiers() {
    return [{
      // right now
      limit: Second,
      refresh: Second,
      message: intl.rightNow
    }, {
      // x seconds
      limit: Minute,
      refresh: Second,
      message: intl.relativeDelta
    }, {
      // x minutes
      limit: Hour,
      refresh: Minute,
      message: intl.relativeDelta
    }, {
      // x hours x minutes
      limit: 12 * Hour,
      refresh: Minute,
      message: intl.relativeDelta
    }, {
      // later/earlier today
      when: ({
        past
      }) => past ? new Daet().reset('hour').getTime() : new Daet().plus(1, 'day').reset('hour').getTime(),
      message: intl.earlierOrLaterToday
    }, {
      // yesterday or tomorrow
      when: ({
        past
      }) => past ? new Daet().minus(1, 'day').reset('hour').getTime() : new Daet().plus(2, 'day').reset('hour').getTime(),
      message: intl.yesterdayOrTommorow
    }, {
      // this week
      when: ({
        past
      }) => past ? new Daet().endOfLastWeek().getTime() : new Daet().startOfNextWeek().getTime(),
      message: intl.relativeThisWeek
    }, {
      // next or last week
      when: ({
        past
      }) => past ? new Daet().endOfLastWeek().endOfLastWeek().getTime() : new Daet().startOfNextWeek().startOfNextWeek().getTime(),
      message: intl.relativeSecondWeek
    }, {
      // earlier or later
      limit: Infinity,
      message: intl.earlierOrLater
    }];
  }
  /** Create a new daet instance based on the input */


  static create(input) {
    return new this(input);
  }
  /** Create a new daet instance based on the input */


  constructor(input) {
    _defineProperty(this, "raw", void 0);

    _defineProperty(this, "getTime", (0, _memo.default)(() => this.raw.getTime()));

    _defineProperty(this, "startOfNextWeek", (0, _memo.default)(() => {
      // continue until we become the start of next week
      let latest = this.plus(1, 'day');

      while (latest.raw.getDay() !== _startOfWeek.default) {
        latest = latest.plus(1, 'day');
      } // reset the time, so we become the start time of that week


      return latest.reset('hour');
    }));

    _defineProperty(this, "endOfLastWeek", (0, _memo.default)(() => {
      // continue until we become the start of this week
      let latest = this;

      while (latest.raw.getDay() !== _startOfWeek.default) {
        latest = latest.minus(1, 'day');
      } // reset the time, then minus a second, so we become the end of the prior week


      return latest.reset('hour').minus(1, 'second');
    }));

    _defineProperty(this, "toISOString", (0, _memo.default)(() => this.raw.toISOString()));

    _defineProperty(this, "toJSON", (0, _memo.default)(() => this.raw.toJSON()));

    this.raw = input instanceof Daet ? new Date(input.raw) : input ? new Date(input) : new Date();
    this.raw.setMilliseconds(0);
  }
  /** Return a new daet instance that is in the past by the value amount. */


  minus(value, unit) {
    return this.plus(value * -1, unit);
  }
  /** Return a new daet instance that is in the future by the value amount. */


  plus(value, unit) {
    switch (unit) {
      case 'millisecond':
        {
          const next = new Date(this.raw.getTime() + value);
          return new Daet(next);
        }

      case 'second':
        {
          return this.plus(value * Second, 'millisecond');
        }

      case 'minute':
        {
          return this.plus(value * Minute, 'millisecond');
        }

      case 'hour':
        {
          return this.plus(value * Hour, 'millisecond');
        }

      case 'day':
        {
          return this.plus(value * Day, 'millisecond');
        }

      case 'week':
        {
          return this.plus(value * Week, 'millisecond');
        }

      default:
        // https://basarat.gitbooks.io/typescript/docs/types/discriminated-unions.html
        const neverCheck = unit;
        throw new Error('unknown unit');
    }
  }
  /** Clone this daet instance based on its raw value. */


  rawClone() {
    return new Date(this.raw);
  }
  /** Clone this daet instance. */


  clone() {
    return new Daet(this);
  }
  /** Return a new daet instance that has the unit set to the desired value. */


  set(value, unit) {
    switch (unit) {
      case 'millisecond':
        {
          const next = this.rawClone().setMilliseconds(value);
          return new Daet(next);
        }

      case 'second':
        {
          const next = this.rawClone().setSeconds(value);
          return new Daet(next);
        }

      case 'minute':
        {
          const next = this.rawClone().setMinutes(value);
          return new Daet(next);
        }

      case 'hour':
        {
          const next = this.rawClone().setHours(value);
          return new Daet(next);
        }

      default:
        // https://basarat.gitbooks.io/typescript/docs/types/discriminated-unions.html
        const neverCheck = unit;
        throw new Error('unknown unit');
    }
  }
  /** Return a new daet instance that has the desired unit reset to 0. */


  reset(unit) {
    switch (unit) {
      case 'millisecond':
        {
          const next = this.rawClone().setMilliseconds(0);
          return new Daet(next);
        }

      case 'second':
        {
          const next = this.rawClone().setSeconds(0, 0);
          return new Daet(next);
        }

      case 'minute':
        {
          const next = this.rawClone().setMinutes(0, 0, 0);
          return new Daet(next);
        }

      case 'hour':
        {
          const next = this.rawClone().setHours(0, 0, 0, 0);
          return new Daet(next);
        }

      default:
        // https://basarat.gitbooks.io/typescript/docs/types/discriminated-unions.html
        const neverCheck = unit;
        throw new Error('unknown unit');
    }
  }
  /** Get the epoch time of this daet instance. */


  /** Get the milliseconds from the passed daet instance to this daet instance. */
  getMillisecondsFrom(from) {
    const now = from.getTime();
    const time = this.getTime();
    return time - now;
  }
  /** Get the milliseconds from now to this daet instance */


  getMillisecondsFromNow() {
    return this.getMillisecondsFrom(new Daet());
  }
  /** Use https://devdocs.io/javascript/global_objects/datetimeformat to format our daet instance. */


  format(locale, options) {
    return new Intl.DateTimeFormat(locale, options).format(this.raw);
  }
  /** Return a new daet instance that is the start of the week proceeding that of this daet instance. */


  /** Get the human absolute date display for this daet instance. */
  calendar() {
    return this.format('en', {
      weekday: 'long',
      year: 'numeric',
      month: 'long',
      day: 'numeric',
      hour: 'numeric',
      minute: 'numeric'
    });
  }
  /** Return the human relative display from now, as well as the millisecond delta before a refresh is needed. */


  fromNowDetails() {
    const now = new Daet();
    const nowTime = now.getTime();
    const eventTime = this.getTime();
    const past = nowTime > eventTime;
    const delta = Math.abs(eventTime - nowTime);
    const tiers = Daet.tiers;
    let lastTierDelta = 0;

    for (const tier of tiers) {
      const limit = tier.limit;
      const when = tier.when ? tier.when({
        past
      }) : 0;
      const tierDelta = limit || Math.abs(when - nowTime);

      if (delta < tierDelta) {
        const message = tier.message({
          past,
          delta,
          when: this
        });
        const refresh = tier.refresh || delta - lastTierDelta;
        return {
          message,
          refresh
        };
      }

      lastTierDelta = tierDelta;
    }

    throw new Error('no tier matched the input delta');
  }
  /** Return the human relative display from now. */


  fromNow() {
    return this.fromNowDetails().message;
  }
  /** Return the ISO string for this daet instance. */


}

exports.default = Daet;